package jdk.management.resource.internal.inst;

import sun.misc.JavaIOFileDescriptorAccess;
import sun.misc.SharedSecrets;
import jdk.internal.instrumentation.InstrumentationMethod;
import jdk.management.resource.ResourceRequest;
import java.io.IOException;
import jdk.management.resource.ResourceRequestDeniedException;
import java.io.FileNotFoundException;
import jdk.management.resource.internal.ApproverGroup;
import jdk.management.resource.internal.ResourceIdImpl;
import java.io.FileDescriptor;
import jdk.internal.instrumentation.InstrumentationTarget;

@InstrumentationTarget("java.io.FileOutputStream")
public final class FileOutputStreamRMHooks
{
    private final FileDescriptor fd;
    private final String path;
    private final Object closeLock;
    private volatile boolean closed;
    
    public FileOutputStreamRMHooks() {
        this.fd = null;
        this.path = null;
        this.closeLock = new Object();
        this.closed = false;
    }
    
    @InstrumentationMethod
    private void open(String s, boolean b) throws FileNotFoundException {
        ResourceIdImpl of = ResourceIdImpl.of(this.path);
        ResourceRequest approver = ApproverGroup.FILE_OPEN_GROUP.getApprover(this);
        long request;
        try {
            request = approver.request(1L, of);
            if (request < 1L) {
                throw new FileNotFoundException(s + ": resource limited: too many open files");
            }
        }
        catch (ResourceRequestDeniedException cause) {
            FileNotFoundException ex = new FileNotFoundException(s + ": resource limited: too many open files");
            ex.initCause(cause);
            throw ex;
        }
        ResourceRequest approver2 = null;
        long request2 = 0L;
        int n = 0;
        try {
            FileDescriptor fd = null;
            try {
                fd = this.getFD();
            }
            catch (IOException ex2) {}
            approver2 = ApproverGroup.FILEDESCRIPTOR_OPEN_GROUP.getApprover(fd);
            try {
                request2 = approver2.request(1L, of);
                if (request2 < 1L) {
                    throw new FileNotFoundException(s + ": resource limited: too many open file descriptors");
                }
            }
            catch (ResourceRequestDeniedException cause2) {
                FileNotFoundException o = new FileNotFoundException(s + ": resource limited: too many open file descriptors");
                o.initCause(cause2);
                throw o;
            }
            this.open(s, b);
            n = 1;
        }
        finally {
            approver2.request(-(request2 - (long)n), of);
            approver.request(-(request - (long)n), of);
        }
    }
    
    @InstrumentationMethod
    public final FileDescriptor getFD() throws IOException {
        return this.getFD();
    }
    
    @InstrumentationMethod
    public void write(int n) throws IOException {
        ResourceIdImpl of = ResourceIdImpl.of(this.path);
        FileDescriptor fd = this.getFD();
        ResourceRequest resourceRequest;
        if (fd == FileDescriptor.err) {
            resourceRequest = ApproverGroup.STDERR_WRITE_GROUP.getApprover(this);
        }
        else if (fd == FileDescriptor.out) {
            resourceRequest = ApproverGroup.STDOUT_WRITE_GROUP.getApprover(this);
        }
        else {
            resourceRequest = ApproverGroup.FILE_WRITE_GROUP.getApprover(this);
        }
        long request;
        try {
            request = resourceRequest.request(1L, of);
            if (request < 1L) {
                throw new IOException("Resource limited: insufficient bytes approved");
            }
        }
        catch (ResourceRequestDeniedException cause) {
            throw new IOException("Resource limited", cause);
        }
        int n2 = 0;
        try {
            this.write(n);
            n2 = 1;
        }
        finally {
            resourceRequest.request(-(request - (long)n2), of);
        }
    }
    
    @InstrumentationMethod
    public void write(byte[] array) throws IOException {
        if (array == null) {
            this.write(array);
            return;
        }
        ResourceIdImpl of = ResourceIdImpl.of(this.path);
        FileDescriptor fd = this.getFD();
        ResourceRequest resourceRequest;
        if (fd == FileDescriptor.err) {
            resourceRequest = ApproverGroup.STDERR_WRITE_GROUP.getApprover(this);
        }
        else if (fd == FileDescriptor.out) {
            resourceRequest = ApproverGroup.STDOUT_WRITE_GROUP.getApprover(this);
        }
        else {
            resourceRequest = ApproverGroup.FILE_WRITE_GROUP.getApprover(this);
        }
        int length = array.length;
        long request;
        try {
            request = resourceRequest.request((long)length, of);
            if (request < (long)length) {
                throw new IOException("Resource limited: insufficient bytes approved");
            }
        }
        catch (ResourceRequestDeniedException cause) {
            throw new IOException("Resource limited", cause);
        }
        int n = 0;
        try {
            this.write(array);
            n = length;
        }
        finally {
            resourceRequest.request(-(request - (long)n), of);
        }
        return;
    }
    
    @InstrumentationMethod
    public void write(byte[] array, int n, int n2) throws IOException {
        if (n2 < 0) {
            this.write(array, n, n2);
            return;
        }
        ResourceIdImpl of = ResourceIdImpl.of(this.path);
        FileDescriptor fd = this.getFD();
        ResourceRequest resourceRequest;
        if (fd == FileDescriptor.err) {
            resourceRequest = ApproverGroup.STDERR_WRITE_GROUP.getApprover(this);
        }
        else if (fd == FileDescriptor.out) {
            resourceRequest = ApproverGroup.STDOUT_WRITE_GROUP.getApprover(this);
        }
        else {
            resourceRequest = ApproverGroup.FILE_WRITE_GROUP.getApprover(this);
        }
        long request;
        try {
            request = resourceRequest.request((long)n2, of);
            if (request < (long)n2) {
                throw new IOException("Resource limited: insufficient bytes approved");
            }
        }
        catch (ResourceRequestDeniedException cause) {
            throw new IOException("Resource limited", cause);
        }
        int n3 = 0;
        try {
            this.write(array, n, n2);
            n3 = n2;
        }
        finally {
            resourceRequest.request(-(request - (long)n3), of);
        }
        return;
    }
    
    @InstrumentationMethod
    public void close() throws IOException {
        synchronized (this.closeLock) {
            if (this.closed) {
                return;
            }
        }
        JavaIOFileDescriptorAccess javaIOFileDescriptorAccess = SharedSecrets.getJavaIOFileDescriptorAccess();
        long handle;
        try {
            handle = javaIOFileDescriptorAccess.getHandle(this.fd);
            if (handle == -1L) {
                handle = (long)javaIOFileDescriptorAccess.get(this.fd);
            }
        }
        catch (UnsupportedOperationException ex) {
            handle = (long)javaIOFileDescriptorAccess.get(this.fd);
        }
        try {
            this.close();
            long handle2;
            try {
                handle2 = javaIOFileDescriptorAccess.getHandle(this.fd);
                if (handle2 == -1L) {
                    handle2 = (long)javaIOFileDescriptorAccess.get(this.fd);
                }
            }
            catch (UnsupportedOperationException ex2) {
                handle2 = (long)javaIOFileDescriptorAccess.get(this.fd);
            }
            ResourceIdImpl of = ResourceIdImpl.of(this.path);
            if (handle2 != handle) {
                ApproverGroup.FILEDESCRIPTOR_OPEN_GROUP.getApprover(this.fd).request(-1L, of);
            }
            ApproverGroup.FILE_OPEN_GROUP.getApprover(this).request(-1L, of);
        }
        finally {
            long handle3;
            try {
                handle3 = javaIOFileDescriptorAccess.getHandle(this.fd);
                if (handle3 == -1L) {
                    handle3 = (long)javaIOFileDescriptorAccess.get(this.fd);
                }
            }
            catch (UnsupportedOperationException ex3) {
                handle3 = (long)javaIOFileDescriptorAccess.get(this.fd);
            }
            ResourceIdImpl of2 = ResourceIdImpl.of(this.path);
            if (handle3 != handle) {
                ApproverGroup.FILEDESCRIPTOR_OPEN_GROUP.getApprover(this.fd).request(-1L, of2);
            }
            ApproverGroup.FILE_OPEN_GROUP.getApprover(this).request(-1L, of2);
        }
    }
}
